/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <wchar.h>

#include <iomanip>
#include <iostream>

#include <stdlib.h>
#include <getopt.h>

#include "libtkcommon/config.h"
#include "libtkcommon/tkintl.h"
#include "libtkanalysis/analyser.h"
#include "libtkanalysis/gnuplot.h"
#include "libtkdb/database.h"

using namespace TkResolver;
using namespace Analysis;

void
print_help ()
{
  std::cout  << PACKAGE_NAME << " version " << PACKAGE_VERSION << std::endl;
  std::wcout << L"Copyright (C) 2005 Robert Ham (rah@bash.sh)" << std::endl;
  std::wcout << std::endl;
  std::wcout << L"This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it" << std::endl;
  std::wcout << L"under the terms of the GNU General Public License, version 2 or later.  See" << std::endl;
  std::wcout << L"the COPYING file that came with this software for details." << std::endl;
  std::wcout << std::endl;
  std::wcout << L" -h, --help                  " << _("Display this help info") << std::endl;
  std::wcout << L" -n, --no-plot               " << _("Don't display gnuplots") << std::endl;
  std::wcout << L" -d, --db-host <host>        " << _("Connect to database on <host>") << std::endl;
  std::wcout << L" -g, --group <name>          " << _("Experiment group name") << std::endl;
  std::wcout << L" -i, --intent <code>         " << _("Intent option code") << std::endl;
  std::wcout << std::endl;
}

int
main (int argc, char ** argv)
{
  TkString group_name("Test group");
  TkString db_host("localhost");
  TkString algorithm;
  bool algorithm_details(false);
  wchar_t * wstr;
  bool plot(true);

  int opt;
  const char * options = "hng:d:";
  struct option long_options[] = {
    { "help",        no_argument,       NULL, 'h' },
    { "no-plot",     no_argument,       NULL, 'n' },
    { "group",       required_argument, NULL, 'g' },
    { "db-host",     required_argument, NULL, 'd' },
    { 0, 0, 0, 0 }
  };
  
  while ((opt = getopt_long (argc, argv, options, long_options, NULL)) != -1) {
    switch (opt) {
      
    case 'g':
      group_name = optarg;
      break;

    case 'd':
      db_host = optarg;
      break;
      
    case 'n':
      plot = false;
      break;
      
    case 'h':
      print_help ();
      exit (0);
      break;
    case ':':
    case '?':
      print_help ();
      exit (1);
      break;
    }
  }
  
  //  try
  //    {
      Db::Database database(db_host);
      Db::Id group_id(database.lookup_experiment_group(group_name));
	  
      std::wcout << _(L"Analysing results for experiment group")
		 << " '"
		 << group_name
		 << "'..."
		 << std::flush;
      
      //      std::wcout << L"default precision was: " << mpf_get_default_prec() << std::endl;
      mpf_set_default_prec(1024);
      Analyser analyser(database, group_id);
	  
      std::wcout << _(L"done (statistics set count: ")
		 << analyser._statistic_sets.size()
		 << _(L")")
		 << std::endl
		 << std::endl;

      
      for (StatisticSets::iterator iter(analyser._statistic_sets.begin());
	   iter != analyser._statistic_sets.end();
	   ++iter)
	{
	  mp_exp_t exponent(100);
	  int base(10);
	  size_t digits(20);
	  std::wcout << L"   === "
		     << TkString(iter->first)
		     << L" ==="
		     << std::endl
	  
	             << L"   "
		     << _(L"n:            ")
		     << TkString(iter->second._n)
		     << std::endl

	             << L"   "
		     << _(L"experiments:  ")
		     << TkString(iter->second._experiment_count)
		     << std::endl

		     << L"   "
		     << _(L"pos min:      ")
		     << TkString(iter->second._position_min)
		     << std::endl
	    
		     << "   "
		     << _(L"pos max:      ")
		     << TkString(iter->second._position_max)
		     << std::endl
	    
		     << "   "
		     << _(L"exp pos mean: ")
	    	     << TkString(iter->second._experiment_position_mean)
		     << std::endl

		     << "   "
		     << _(L"exp pos var:  ")
	    	     << TkString(iter->second._experiment_position_variance)
		     << std::endl
	    ;
	  
	  std::wcout << "   "
		     << _(L"Offsets:      ")
		     << std::endl;
	  for (OffsetStatisticsMap::iterator offset_iter( iter->second._offsets._offset_stats.begin() );
	       offset_iter != iter->second._offsets._offset_stats.end();
	       ++offset_iter)
	    {
	      std::wcout << L"      "
			 << offset_iter->first
			 << L":"
			 << std::endl

			 << L"         "
			 << L"p:    "
			 << TkString( analyser._offset_probabilities[offset_iter->first] )
			 << std::endl

			 << L"         "
			 << L"mean: "
			 << TkString( offset_iter->second._mean )
			 << std::endl

			 << L"         "
			 << L"var:  "
			 << TkString( offset_iter->second._variance )
			 << std::endl

			 << L"         "
			 << L"skew: "
			 << TkString( offset_iter->second._skewness )
			 << std::endl
		
			 << L"         "
			 << L"rss:  "
			 << TkString( offset_iter->second._rss )
			 << std::endl
		;
	    }
	  
	  if (plot)
	    Gnuplot plot(iter->first, iter->second._offsets);
	  
	  StatisticSets::iterator inner_iter(iter);
	  for (++inner_iter;
	       inner_iter != analyser._statistic_sets.end();
	       ++inner_iter)
	    {

	      if (plot)
		Gnuplot plot(iter->first, iter->second._offsets,
			     inner_iter->first, inner_iter->second._offsets);
	      
	      if (iter->second._experiment_count > 1
		  && inner_iter->second._experiment_count > 1)
		{
		  TTest t(iter->second, inner_iter->second);
		  std::wcout << L"   "
			     << TkString(iter->first) + L"-" + TkString(inner_iter->first)
			     << L" t-test:"
			     << std::endl
		    
			     << L"      "
			     << _(L"t score: ")
			     << TkString(t._t)
			     << std::endl
		    
			     << L"      "
			     << _(L"df:      ")
			     << TkString(t._df)
			     << std::endl
		    
			     << L"      "
			     << _(L"p:       ")
			     << TkString(t._p)
			     << std::endl

			     << L"      "
			     << _(L"α:       ")
			     << TkString(static_cast<Real>(100 *(1 - t._p)), 3)
			     << L"%"
			     << std::endl
		    ;
		}
	    }
	  
	  std::wcout << std::endl;
	  
	}
      /*    } 
   catch (const NotEnoughResults & ner)
     {
       std::wcout << _(L"not enough results")
		  << std::endl;
       exit(0);
     }
   catch (const Db::Error & e)
     {
       std::wcerr << std::endl
		  << _(L"Database error: ")
		  << TkString( e.what() )
		  << L") - aborting"
		  << std::endl;
       exit(1);
     }
   catch (const mysqlpp::BadQuery& e)
     {
       std::wcerr << std::endl
		  << _(L"Bad database query: ")
		  << TkString( e.what() )
		  << std::endl;
       exit(1);
     }
   catch (const mysqlpp::Exception& e)
     {
       std::wcerr << std::endl
		  << _(L"Unknown database error: ")
		  << TkString( e.what() )
		  << std::endl;
       exit(1);
       }*/

  exit (0);
}
