/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_ANALYSER_H__
#define __TK_ANALYSER_H__

#include "gmpxx.h"

#include "libtkcommon/numerical-types.h"
#include "libtkcommon/tkstring.h"
#include "libtkcommon/pair.h"
#include "libtkcommon/exceptions.h"
#include "libtkalgorithm/algorithm.h"
#include "libtkdb/database.h"
#include "libtkresolver/types.h"

namespace TkResolver {
  namespace Analysis {

    typedef Db::Code IntentCode;
    
    typedef std::list<Offset> OffsetList;
    typedef std::map<Offset, Count> OffsetDistribution;
    typedef std::map<Position, OffsetDistribution> PositionOffsetDistributions;
    typedef std::map<Position, Count> PositionCounts;
    
    class PositionFunction : public std::map<Position, Rational>
      {
      public:
	const Rational* operator()(const Position &pos) const;
        Rational calc_rss(const Probability &prob) const;
	Rational calc_mean() const;
	Rational calc_variance(const Rational &mean) const;
	Real     calc_skewness(const Rational &mean, const Rational &variance) const;
      };
    
    class OffsetStatistics
      {
      public:
	Probability              _probability;
	PositionCounts           _position_counts;
	PositionFunction         _position_function;
	Rational                 _rss;
	Rational                 _mean;
	Rational                 _variance;
	Real                     _skewness;
      };
    typedef std::map<Offset, OffsetStatistics> OffsetStatisticsMap;
    
    class OffsetStatisticsSet
      {
      public:
	OffsetList               _offsets;
	OffsetStatisticsMap      _offset_stats;
	PositionCounts           _position_totals;
	
	OffsetStatisticsSet() {}
	OffsetStatisticsSet(const OffsetProbabilities &probs,
			    const PositionOffsetDistributions &offset_distributions,
			    const PositionCounts &position_counts);
      };
    
    class ExperimentStatistics
      {
      public:
        Position                 _position_mean;
      };
    typedef std::list<ExperimentStatistics> ExperimentStatisticsSet;
    
    class Statistics
      {
      public:
	Count                    _n;
	Position                 _position_min;
	Position                 _position_max;
	
	OffsetStatisticsSet      _offsets;
	ExperimentStatisticsSet  _experiments;
	
	Count                    _experiment_count;
	Position                 _experiment_position_mean;
	Rational                 _experiment_position_variance;
	
	Statistics(Db::Database &database,
		   const OffsetProbabilities &offset_probs,
		   const Db::IdList &experiments);
      };
    
    class TTest
      {
      private:
      public:
	Real     _t;
	Integer  _df;
	Real     _p;
	
	TTest(const Statistics &stats_a, const Statistics &stats_b);
      };
    
    typedef std::map<IntentCode, Statistics> StatisticSets;
    
    class Analyser
      {
      public:
	StatisticSets _statistic_sets;
	OffsetProbabilities _offset_probabilities;
	
	Analyser(Db::Database &database, const Db::Id &experiment_group);
      };

    
    class NotEnoughResults : std::runtime_error
      {
      public:
	NotEnoughResults()
	  : std::runtime_error("not enough results") {}
      };

    
  }; /* namespace Analysis */
  
  double gamma(double x);
}; /* namespace TkResolver */


#endif /* __TK_ANALYSER_H__ */

