/* -*- c++-mode -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_DATABASE_H__
#define __TK_DATABASE_H__

#define GETTEXT_PACKAGE "TkResolver"

#include <stdexcept>

#include <mysql++/mysql++.h>

#include "database_types.h"
#include "libtkcommon/tkstring.h"
#include "libtkrandom/random_types.h"
#include "libtkresolver/types.h"

#define EXPERIMENT_GROUP "Test group"

namespace TkResolver {
  namespace Db {
    
    String date_time_string (time_t t);
    
    class Error : public std::runtime_error
      {
      public:
	Error (mysqlpp::Connection &connection);
      };
  
    class Database
      {
      private:
	typedef std::map<Name, Type> FieldInfo;
	typedef std::map<Name, FieldInfo> TableInfo;

	mysqlpp::Connection _connection;
	
	mysqlpp::Query *    _table_info_query;
	TableInfo           _table_info;

	void create_table_info_query();
	void get_table_info(const Name &name);
	void check_table_info(const Name &name);

	Id insert_experiment(const Id &experiment_group,
			     const Db::CodeValue &intent_option_code,
			     const time_t &date_time,
			     const TkString &location_name,
			     const TkString &notes,
			     const TkDouble &ability_rating,
			     unsigned width,
			     unsigned height,
			     const TkString &retriever,
			     bool retriever_is_true_random,
			     bool retreiver_specified,
			     const Random::Bits &randomness);
	void insert_experiment(const Id &experiment_id,
			       const IdList &operators);
	void insert_experiment(const Id &experiment_id,
			       const Result &result);
      public:
	Database(const TkString &hostname = "localhost",
		 const TkString &database = "psi",
		 const TkString &username = "psi",
		 const TkString &password = "isp");
	~Database();

	Type      get_column_type(const Name &table_name, const Name &column_name);
	DataSet * get_table_data(const Name &table_name, const NameList &column_names);
	Id insert_table_data(const Name &table_name, const Parameters &params);

	Db::Id lookup_experiment_group(const Name &name);
	void get_experiment_group_parameters(const Id &experiment_group,
					     TkString *algorithm_spec,
					     unsigned *bit_descriptor = 0,
					     unsigned short *traces = 0,
					     unsigned *total_time = 0,
					     double *speed = 0,
					     double *horizon = 0);
	Id       choose_experiment_group();
	TkString choose_location();
	Id       choose_operator();
	Code     choose_intent_option(const Id &experiment_group);
	Codes        get_intent_options(const Id &experiment_group);
	IdList       get_experiments(const Id &experiment_group, const Code &intent_option_code);
	unsigned     get_experiment_screen_width(const Id &experiment);
	DataCursor * get_results(const Id &experiment);
	
	Id insert_experiment(const Id &experiment_group,
			     const Db::CodeValue &intent_option_code,
			     const time_t &date_time,
			     const TkString &location_name,
			     const TkString &notes,
			     const TkDouble &ability_rating,
			     unsigned width,
			     unsigned height,
			     const TkString &retriever,
			     bool retriever_is_true_random,
			     bool retriever_specified,
			     const Random::Bits &randomness,
			     const IdList &operators,
			     const Result &result);
      };

  }; /* namespace Db */
}; /* namespace TkResolver */

#endif /* __TK_DATABASE_H__ */
