/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

template<typename S>
NameList
display_column_names(const std::list< DisplayColumnInfo<S> > & display_columns)
{
  NameList columns;
  bool exists;
  for (typename std::list< DisplayColumnInfo<S> >::const_iterator iter(display_columns.begin());
       iter != display_columns.end();
       ++iter)
    {
      for (NameList::const_iterator dciter(iter->columns.begin());
	   dciter != iter->columns.end();
	   ++dciter)
	{
	  columns.push_back(*dciter);
	}
    }

  return columns;
}

template<typename C>
Gtk::CellRenderer *
DataSelector:: create_view_cell(const DisplayColumnInfo<typename C::SlotCellData> & display_column, DisplayColumnAdaptor<C> & adaptor)
{
  Gtk::CellRenderer * cell;
  if (display_column.slot != 0) /* have slot */
    {
      DEBUG_OUT << "callback column '" << display_column.title << "'" << std::endl;

      cell = Gtk::manage(new Gtk::CellRendererText);
      adaptor.pack_start(*cell);
      adaptor.set_cell_data_func(*cell, *display_column.slot);
    }
  else /* don't have slot; just pack renderers into column */
    {
      DEBUG_OUT << "packing cells for column '" << display_column.title << "'" << std::endl;

      for (NameList::const_iterator iter( display_column.columns.begin() );
	   iter != display_column.columns.end();
	   ++iter)
	{
	  DEBUG_OUT << "packing cell '" << *iter << "' for column '" << display_column.title << "'" << std::endl;

	  cell = create_cell(*iter);
	  adaptor.pack_start(*cell);
	  adaptor.add_attribute(*cell, get_column_view_attribute(*iter), get_column_index(*iter));
	}
    }

  return cell;
}



template<typename C>
DisplayColumnAdaptor<C>:: DisplayColumnAdaptor(C * column)
  : _column(column)
{
}

template<typename C>
void
DisplayColumnAdaptor<C>:: pack_start(Gtk::CellRenderer & cell)
{
  _column->pack_start(cell);
}

template<typename C>
void
DisplayColumnAdaptor<C>:: set_cell_data_func(Gtk::CellRenderer & cell, typename C::SlotCellData & slot)
{
  _column->set_cell_data_func(cell, slot);
}

template<typename C>
void
DisplayColumnAdaptor<C>:: add_attribute(Gtk::CellRenderer & cell, const Glib::ustring & attribute, int column_index)
{
  _column->add_attribute(cell, attribute, column_index);
}
