/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#include "data_selector_list.h"

using namespace TkResolver;
using namespace Db::Ui;

void
TkResolver::Db::Ui:: append_selection(const Gtk::TreeModel::iterator & iter, ModelIteratorList * list)
{
  list->push_back(iter);
}

/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

DataSelectorList:: DataSelectorList(Db::Database * database,
					const Name & table_name,
					const NameList & selection_columns,
					const Name & title)
  : DataSelector (database, table_name, selection_columns),
    Gtk::Frame(title)
{
  add(_box);

  //
  // Data view
  //
  _view_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  _box.pack_start(_view_scroll, true, true);
  _view_scroll.set_shadow_type(Gtk::SHADOW_IN);
}


void 
DataSelectorList:: set_display_columns(const ListColumnList & display_columns)
{
  //
  // Create the data models
  //
  set_columns(display_column_names(display_columns));
  
  _view.set_model(get_store());
  if (display_columns.size() == 1)
    _view.set_headers_visible(false);

  for (ListColumnList::const_iterator iter(display_columns.begin());
       iter != display_columns.end();
       ++iter)
    {
      Gtk::TreeViewColumn * column = Gtk::manage(new Gtk::TreeViewColumn(iter->title));
      ListColumnAdaptor adaptor(column);
      create_view_cell(*iter, adaptor);
      _view.append_column(*column);
    }
  _view_scroll.add(_view);
  
  show_all_children();
}

Gtk::TreeView &
DataSelectorList:: get_view()
{
  return _view;
}

Gtk::HBox &
DataSelectorList:: get_box()
{
  return _box;
}

ModelIteratorList
DataSelectorList:: get_selected() const
{
  const Glib::RefPtr<const Gtk::TreeSelection> sel(_view.get_selection());
  
  if (sel->count_selected_rows() < 1)
    {
      throw NoSelection();
    }

  ModelIteratorList list;
  sel->selected_foreach_iter( sigc::bind(sigc::ptr_fun(&append_selection), &list) );

  return list;
}

void
DataSelectorList:: select(Gtk::TreeModel::iterator &iter)
{
  _view.get_selection()->select(iter);
}
