/* -*- c++ -*-
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so by
 * e-mail - mail your message to Viktor T. Toth at <vttoth@vttoth.com>.
 *
 * Updated 2007 by Robert Ham for tkresolver
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh> for tkresolver
 * Copyright (C) Viktor T. Toth at <vttoth@vttoth.com>
 *
 */

#ifndef __TK_MP_REAL_H__
#define __TK_MP_REAL_H__

#include <gmpxx.h>

namespace TkResolver {
  namespace MP {
    
    template <typename T>
    class Matrix;
    
    namespace Real {
      
      typedef mpf_class Number;
      
      typedef MP::Matrix<Number> Matrix;
      
      class Pi : public Number
      {
      public:
	Pi(unsigned long prec = 1024);
	void set(unsigned long prec);
      };
      
      class E : public Number
      {
      public:
	E(unsigned long prec = 1024);
	void set(unsigned long prec);
      };
      
      extern Pi pi;
      extern E  e;
      
      Number pow(const Number &op1, unsigned long op2);
      Number pow(const Number &op1, const Number &op2);
      Number log(const Number &x);
      Number sin(const Number &x);
      Number cos(const Number &x);
      Number exp(const Number &x);
      Number atan2(const Number &y, const Number &x);
    } /* namespace Real */
  } /* namespace MP */
} /* namespace TkResolver */




#endif /* __TK_MP_REAL_H__ */
