/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * GPL all the way, baby :)
 *
 */

#ifndef __TK_DISPLAY_H__
#define __TK_DISPLAY_H__

#include <list>
#include <vector>

#include <SDL/SDL.h>

#include "libtkcommon/pair.h"
#include "types.h"

namespace TkResolver {

  class Display
  {
  private:
    typedef std::list<SDL_Rect> BlockList;
    typedef std::vector<Line>   LineList;
    
    SDL_Surface  * _sdl_display;
    Uint32         _white;
    Uint32         _black;
    Uint32         _red;
    unsigned short _ntraces;
    unsigned       _total_lines;
    unsigned       _print_count;
    Line           _start_x;
    unsigned       _last_y;
    bool           _scrolling;
    unsigned       _horizon;
    LineList     * _x_history;

    int truncate_x(int x);
    inline const int& get_last_x(OffsetIndex index) const;
    void get_x_limits(OffsetIndex index, int &min, int &max);
    
    void lock();
    void unlock();
    BlockList create_count_down_blocks(unsigned count);
  public:
    static unsigned calc_total_lines(unsigned total_time, double speed, unsigned height);
    
    Display (unsigned height,
	     unsigned width,
	     unsigned short ntraces,
	     unsigned total_time,
	     double   speed = 0.0,
	     double   horizon = 0.0);
    ~Display();
  
    void count_down (unsigned count);
    void print_line (const Line &line);

    const Line & get_start_line() const;
    unsigned get_height() const;
    unsigned get_width() const;
    unsigned get_total_lines() const;
  };

}; /* namespace TkResolver */

#endif /* __TK_DISPLAY_H__ */
